'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var url = require('url');
var events = require('events');
var WebSocket = _interopDefault(require('ws'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var realFetch = require('node-fetch');
module.exports = function(url, options) {
	if (/^\/\//.test(url)) {
		url = 'https:' + url;
	}
	return realFetch.call(this, url, options);
};

if (!global.fetch) {
	global.fetch = module.exports;
	global.Response = realFetch.Response;
	global.Headers = realFetch.Headers;
	global.Request = realFetch.Request;
}

/**
 * @hidden
 */
var Streaming = /** @class */ (function (_super) {
    __extends(Streaming, _super);
    /**
     *
     * @param apiURL REST api url см [документацию](https://tinkoffcreditsystems.github.io/invest-openapi/env/)
     * @param socketURL Streaming api url см [документацию](https://tinkoffcreditsystems.github.io/invest-openapi/env/)
     * @param secretToken токен доступа см [получение токена доступа](https://tinkoffcreditsystems.github.io/invest-openapi/auth/)
     *
     */
    function Streaming(_a) {
        var url = _a.url, secretToken = _a.secretToken;
        var _this = _super.call(this) || this;
        _this._ws = null;
        _this._wsQueue = [];
        _this._subscribeMessages = [];
        /**
         * Обработчик открытия соединения
         */
        _this.handleSocketOpen = function () {
            _this.emit('socket-open');
            _this.dispatchWsQueue();
        };
        /**
         * Обработчик серверного пинга
         */
        _this.handleSocketPing = function (m) {
            var _a;
            (_a = _this._ws) === null || _a === void 0 ? void 0 : _a.pong(m);
            clearTimeout(_this._wsPingTimeout);
            _this._wsPingTimeout = setTimeout(function () {
                var _a;
                (_a = _this._ws) === null || _a === void 0 ? void 0 : _a.terminate();
            }, 30000 + 1000);
        };
        /**
         * Обработчик закрытия соединения
         */
        _this.handleSocketClose = function () {
            clearTimeout(_this._wsPingTimeout);
            _this.emit('socket-close');
            _this.handleSocketError();
        };
        /**
         * Обработчик ошибок и переподключение при необходимости
         */
        _this.handleSocketError = function () {
            var _a, _b;
            clearTimeout(_this._wsPingTimeout);
            var isClosed = [2 /* CLOSING */, 3 /* CLOSED */].includes((_a = _this._ws) === null || _a === void 0 ? void 0 : _a.readyState);
            if (isClosed) {
                (_b = _this._ws) === null || _b === void 0 ? void 0 : _b.terminate();
                _this._ws = null;
                _this.connect();
            }
        };
        /**
         * Обработчик входящих сообщений
         */
        _this.handleSocketMessage = function (m) {
            var _a = JSON.parse(m), type = _a.event, payload = _a.payload;
            _this.emit(_this.getEventName(type, payload), payload);
        };
        _this.socketURL = url;
        _this.secretToken = secretToken;
        _this.authHeaders = {
            Authorization: 'Bearer ' + _this.secretToken,
            'Content-Type': 'application/json',
        };
        return _this;
    }
    /**
     * Соединяемся с сокетом
     */
    Streaming.prototype.connect = function () {
        var _this = this;
        if (this._ws && [1 /* OPEN */, 0 /* CONNECTING */].includes(this._ws.readyState)) {
            return;
        }
        this._ws = new WebSocket(this.socketURL, {
            perMessageDeflate: false,
            headers: this.authHeaders,
        });
        this._ws.on('open', this.handleSocketOpen);
        this._ws.on('ping', this.handleSocketPing);
        this._ws.on('message', this.handleSocketMessage);
        this._ws.on('close', this.handleSocketClose);
        this._ws.on('error', this.handleSocketError);
        // Восстанавливаем подписки
        if (this._subscribeMessages) {
            this._subscribeMessages.forEach(function (msg) {
                _this.enqueue(msg);
            });
        }
    };
    /**
     * Получение имени ивента
     */
    Streaming.prototype.getEventName = function (type, params) {
        if (type === 'orderbook') {
            return type + "-" + params.figi + "-" + params.depth;
        }
        if (type === 'candle') {
            return type + "-" + params.figi + "-" + params.interval;
        }
        if (type === 'instrument_info') {
            return type + "-" + params.figi;
        }
        throw new Error("Unknown type: " + type);
    };
    /**
     * Поставить сообщение в очередь на отправку в сокет
     */
    Streaming.prototype.enqueue = function (command) {
        this._wsQueue.push(command);
        this.dispatchWsQueue();
    };
    /**
     * Разбор очереди сообщений на отправку в сокет
     */
    Streaming.prototype.dispatchWsQueue = function () {
        var _this = this;
        var _a;
        if (((_a = this._ws) === null || _a === void 0 ? void 0 : _a.readyState) === 1 /* OPEN */) {
            var cb = function () { return _this._wsQueue.length && _this.dispatchWsQueue(); };
            this._ws.send(JSON.stringify(this._wsQueue.shift()), cb);
        }
    };
    /**
     * Подписка на различные каналы в сокете
     */
    Streaming.prototype.subscribeToSocket = function (_a, cb) {
        var _this = this;
        var type = _a.type, params = __rest(_a, ["type"]);
        if (!this._ws) {
            this.connect();
        }
        var message = __assign({ event: type + ":subscribe" }, params);
        this.enqueue(message);
        this._subscribeMessages.push(message);
        var handler = function (x) { return cb(x); };
        var eventName = this.getEventName(type, params);
        this.on(eventName, handler);
        return function () {
            _this.off(eventName, handler);
            if (!_this.listenerCount(eventName)) {
                _this.enqueue(__assign({ event: type + ":unsubscribe" }, params));
                var index = _this._subscribeMessages.findIndex(function (msg) { return msg === message; });
                if (index !== -1) {
                    _this._subscribeMessages.splice(index, 1);
                }
            }
        };
    };
    Streaming.prototype.orderbook = function (_a, cb) {
        var figi = _a.figi, _b = _a.depth, depth = _b === void 0 ? 3 : _b;
        if (cb === void 0) { cb = console.log; }
        return this.subscribeToSocket({ type: 'orderbook', figi: figi, depth: depth }, cb);
    };
    /**
     * Метод для подписки на данные по свечному графику инструмента
     * @example см. метод [[orderbook]]
     * @param figi идентификатор инструмента
     * @param interval интервал для свечи
     * @param cb функция для обработки новых данных по свечи
     * @return функция для отмены подписки
     */
    Streaming.prototype.candle = function (_a, cb) {
        var figi = _a.figi, _b = _a.interval, interval = _b === void 0 ? '1min' : _b;
        if (cb === void 0) { cb = console.log; }
        return this.subscribeToSocket({ type: 'candle', figi: figi, interval: interval }, cb);
    };
    /**
     * Метод для подписки на данные по инструменту
     * @example см. метод [[orderbook]]
     * @param figi идентификатор инструмента
     * @param cb функция для обработки новых данных по инструменту
     * @return функция для отмены подписки
     */
    Streaming.prototype.instrumentInfo = function (_a, cb) {
        var figi = _a.figi;
        if (cb === void 0) { cb = console.log; }
        return this.subscribeToSocket({ type: 'instrument_info', figi: figi }, cb);
    };
    return Streaming;
}(events.EventEmitter));

function getQueryString(params) {
    params = Object.fromEntries(Object.entries(params).filter(function (_a) {
        var _ = _a[0], value = _a[1];
        return value != null;
    }));
    // must be a number https://github.com/microsoft/TypeScript/issues/32951
    var searchParams = new url.URLSearchParams(params).toString();
    return searchParams.length ? "?" + searchParams : '';
}
var OpenAPI = /** @class */ (function () {
    /**
     *
     * @param apiURL REST api url см [документацию](https://tinkoffcreditsystems.github.io/invest-openapi/env/)
     * @param socketURL Streaming api url см [документацию](https://tinkoffcreditsystems.github.io/invest-openapi/env/)
     * @param secretToken токен доступа см [получение токена доступа](https://tinkoffcreditsystems.github.io/invest-openapi/auth/)
     *
     */
    function OpenAPI(_a) {
        var apiURL = _a.apiURL, socketURL = _a.socketURL, secretToken = _a.secretToken;
        this._sandboxCreated = false;
        this._streaming = new Streaming({ url: socketURL, secretToken: secretToken });
        this.apiURL = apiURL;
        this.secretToken = secretToken;
        this.authHeaders = {
            Authorization: 'Bearer ' + this.secretToken,
            'Content-Type': 'application/json',
        };
    }
    /**
     * Запрос к REST
     */
    OpenAPI.prototype.makeRequest = function (url, _a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.method, method = _c === void 0 ? 'get' : _c, params = _b.params;
        return __awaiter(this, void 0, void 0, function () {
            var requestParams, requestUrl, res, _d, data;
            return __generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        requestParams = { method: method, headers: new Headers(this.authHeaders) };
                        requestUrl = method === 'get' ? this.apiURL + url + getQueryString(params || {}) : this.apiURL + url;
                        if (method === 'post') {
                            requestParams.body = JSON.stringify(params);
                        }
                        return [4 /*yield*/, fetch(requestUrl, requestParams)];
                    case 1:
                        res = _e.sent();
                        if (res.status === 401) {
                            return [2 /*return*/, Promise.reject(new Error('Unauthorized! Try to use valid token. https://tinkoffcreditsystems.github.io/invest-openapi/auth/'))];
                        }
                        if (!!res.ok) return [3 /*break*/, 6];
                        if (!(res.headers.get('content-type') === 'application/json')) return [3 /*break*/, 3];
                        return [4 /*yield*/, res.json()];
                    case 2:
                        _d = (_e.sent()).payload;
                        return [3 /*break*/, 5];
                    case 3: return [4 /*yield*/, res.text()];
                    case 4:
                        _d = _e.sent();
                        _e.label = 5;
                    case 5: throw _d;
                    case 6: return [4 /*yield*/, res.json()];
                    case 7:
                        data = _e.sent();
                        return [2 /*return*/, data.payload];
                }
            });
        });
    };
    /**
     * Регистрация песочницы
     */
    OpenAPI.prototype.sandboxRegister = function () {
        if (!this._sandboxCreated) {
            this.makeRequest('/sandbox/register', { method: 'post' });
            this._sandboxCreated = true;
        }
    };
    /**
     * Метод для очистки песочницы
     */
    OpenAPI.prototype.sandboxClear = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.sandboxRegister()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.makeRequest('/sandbox/clear', { method: 'post' })];
                }
            });
        });
    };
    /**
     * Метод для задания баланса по бумагам
     * @param params см. описание типа
     */
    OpenAPI.prototype.setPositionBalance = function (params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.sandboxRegister()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.makeRequest('/sandbox/positions/balance', { method: 'post', params: params })];
                }
            });
        });
    };
    /**
     * Метод для задания баланса по валютам
     * @param params см. описание типа
     */
    OpenAPI.prototype.setCurrenciesBalance = function (params) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.sandboxRegister()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.makeRequest('/sandbox/currencies/balance', { method: 'post', params: params })];
                }
            });
        });
    };
    /**
     * Метод для получение портфеля цб
     */
    OpenAPI.prototype.portfolio = function () {
        return this.makeRequest('/portfolio');
    };
    /**
     * Метод для получения валютных активов клиента
     */
    OpenAPI.prototype.portfolioCurrencies = function () {
        return this.makeRequest('/portfolio/currencies');
    };
    /**
     * Метод для получение данных по инструменту в портфеле
     */
    OpenAPI.prototype.instrumentPortfolio = function (params) {
        return this.portfolio().then(function (x) {
            return (x.positions.find(function (position) {
                if ('figi' in params) {
                    return position.figi === params.figi;
                }
                if ('ticker' in params) {
                    return position.ticker === params.ticker;
                }
            }) || null);
        });
    };
    /**
     * Метод для выставления заявки
     * @param figi идентификатор инструмента
     * @param lots количество лотов для заявки
     * @param operation тип заявки
     * @param price цена лимитной заявки
     */
    OpenAPI.prototype.limitOrder = function (_a) {
        var figi = _a.figi, lots = _a.lots, operation = _a.operation, price = _a.price;
        return this.makeRequest("/orders/limit-order?figi=" + figi, {
            method: 'post',
            params: {
                lots: lots,
                operation: operation,
                price: price,
            },
        });
    };
    /**
     * Метод для выставления заявки
     * @param figi идентификатор инструмента
     * @param lots количество лотов для заявки
     * @param operation тип заявки
     * @param price цена лимитной заявки
     */
    OpenAPI.prototype.marketOrder = function (_a) {
        var figi = _a.figi, lots = _a.lots, operation = _a.operation;
        return this.makeRequest("/orders/market-order?figi=" + figi, {
            method: 'post',
            params: {
                lots: lots,
                operation: operation,
            },
        });
    };
    //todo протестить
    /**
     * Метод для отмены активных заявок
     * @param orderId идентифткатор заявки
     */
    OpenAPI.prototype.cancelOrder = function (_a) {
        var orderId = _a.orderId;
        return this.makeRequest("/orders/cancel?orderId=" + orderId, { method: 'post' });
    };
    /**
     * Метод для получения всех активных заявок
     */
    OpenAPI.prototype.orders = function () {
        return this.makeRequest('/orders');
    };
    /**
     * Метод для получения всех доступных валютных инструментов
     */
    OpenAPI.prototype.currencies = function () {
        return this.makeRequest('/market/currencies');
    };
    /**
     * Метод для получения всех доступных валютных ETF
     */
    OpenAPI.prototype.etfs = function () {
        return this.makeRequest('/market/etfs');
    };
    /**
     * Метод для получения всех доступных облигаций
     */
    OpenAPI.prototype.bonds = function () {
        return this.makeRequest('/market/bonds');
    };
    /**
     * Метод для получения всех доступных акций
     */
    OpenAPI.prototype.stocks = function () {
        return this.makeRequest('/market/stocks');
    };
    /**
     * Метод для получения операций по цб по инструменту
     * @param from Начало временного промежутка в формате ISO 8601
     * @param to Конец временного промежутка в формате ISO 8601
     * @param figi Figi-идентификатор инструмента
     */
    OpenAPI.prototype.operations = function (_a) {
        var from = _a.from, to = _a.to, figi = _a.figi;
        return this.makeRequest('/operations', {
            params: { from: from, to: to, figi: figi },
        });
    };
    /**
     * Метод для получения исторических свечей по FIGI
     * @param from Начало временного промежутка в формате ISO 8601
     * @param to Конец временного промежутка в формате ISO 8601
     * @param figi Figi-идентификатор инструмента
     * @param interval интервал для свечи
     */
    OpenAPI.prototype.candlesGet = function (_a) {
        var from = _a.from, to = _a.to, figi = _a.figi, _b = _a.interval, interval = _b === void 0 ? '1min' : _b;
        return this.makeRequest('/market/candles', {
            params: { from: from, to: to, figi: figi, interval: interval },
        });
    };
    /**
     * Метод для получение стакана
     * @param figi Figi-идентификатор инструмента
     * @param depth
     */
    OpenAPI.prototype.orderbookGet = function (_a) {
        var figi = _a.figi, _b = _a.depth, depth = _b === void 0 ? 3 : _b;
        return this.makeRequest('/market/orderbook', {
            params: { figi: figi, depth: depth },
        });
    };
    /**
     * Метод для поиска инструментов по figi или ticker
     * @param params { figi или ticker }
     */
    OpenAPI.prototype.search = function (params) {
        if ('figi' in params) {
            return this.makeRequest('/market/search/by-figi', {
                params: { figi: params.figi },
            }).then(function (x) { return (x ? { total: 1, instruments: [x] } : { total: 0, instruments: [] }); });
        }
        if ('ticker' in params) {
            return this.makeRequest('/market/search/by-ticker', {
                params: { ticker: params.ticker },
            });
        }
        throw new Error('should specify figi or ticker');
    };
    /**
     * Метод для поиска инструмента по figi или ticker
     * @param params { figi или ticker }
     */
    OpenAPI.prototype.searchOne = function (params) {
        return this.search(params).then(function (x) { return x.instruments[0] || null; });
    };
    /**
     * Метод для подписки на данные по стакану инструмента
     * @example
     * ```typescript
     * const { figi } = await api.searchOne({ ticker: 'AAPL' });
     * const unsubFromAAPL = api.orderbook({ figi }, (ob) => { console.log(ob.bids) });
     * // ... подписка больше не нужна
     * unsubFromAAPL();
     * ```
     * @param figi идентификатор инструмента
     * @param depth
     * @param cb функция для обработки новых данных по стакану
     * @return функция для отмены подписки
     */
    OpenAPI.prototype.orderbook = function (_a, cb) {
        var figi = _a.figi, _b = _a.depth, depth = _b === void 0 ? 3 : _b;
        if (cb === void 0) { cb = console.log; }
        return this._streaming.orderbook({ figi: figi, depth: depth }, cb);
    };
    /**
     * Метод для подписки на данные по свечному графику инструмента
     * @example см. метод [[orderbook]]
     * @param figi идентификатор инструмента
     * @param interval интервал для свечи
     * @param cb функция для обработки новых данных по свечи
     * @return функция для отмены подписки
     */
    OpenAPI.prototype.candle = function (_a, cb) {
        var figi = _a.figi, _b = _a.interval, interval = _b === void 0 ? '1min' : _b;
        if (cb === void 0) { cb = console.log; }
        return this._streaming.candle({ figi: figi, interval: interval }, cb);
    };
    /**
     * Метод для подписки на данные по инструменту
     * @example см. метод [[orderbook]]
     * @param figi идентификатор инструмента
     * @param cb функция для обработки новых данных по инструменту
     * @return функция для отмены подписки
     */
    OpenAPI.prototype.instrumentInfo = function (_a, cb) {
        var figi = _a.figi;
        if (cb === void 0) { cb = console.log; }
        return this._streaming.instrumentInfo({ figi: figi }, cb);
    };
    return OpenAPI;
}());

module.exports = OpenAPI;
